#include "ccapteurtemhum_si7021_a20.h"

float CCapteurTemHum_SI7021_A20::lireMesureHum()
{

  float hum;
  unsigned char lecture[3];
  unsigned char ecriture[1];
  int res;
  ecriture[0] = COM_MES_HUM;
  mI2c->ecrire(ADR, ecriture, 1);
  usleep(100000);
  res=mI2c->lire(ADR, lecture, 2);
  if (res != 2) {
      QString mess="CCapteurTemHum_SI7021_A20::lireMesureHum ERREUR Lecture";
      emit sigErreur(mess);
      return -1;
  } // if res
  unsigned char MSB = lecture[0];
  unsigned char LSB = lecture[1]&0xF0;
  hum=((MSB<<8)+LSB);
  hum = -6+125*hum/65536;
  return hum;
}

float CCapteurTemHum_SI7021_A20::lireMesureTem()
{
  float tem;
      unsigned char lecture[2];
      unsigned char ecriture[1];
      int res;
      ecriture[0] = COM_MES_TEMP;
      mI2c->ecrire(ADR, ecriture, 1);
      usleep(100000);
      res=mI2c->lire(ADR, lecture, 2);
      if (res != 2) {
          QString mess="CCapteurTemHum_SI7021_A20::lireMesureTem ERREUR Lecture";
          emit sigErreur(mess);
          return -1;
       } // if res
      unsigned char MSB = lecture[0];
      unsigned char LSB = lecture[1]&0xFC;
      tem = ((MSB<<8)+LSB);
      tem = -46.85+175.72*tem/65536;
      return tem;
}

void CCapteurTemHum_SI7021_A20::run()
{
  float mesureHum, mesureTem;
       while(!m_fin) {
           // écriture de la mesure dans le segment de mémoire partagé
           mesureHum = lireMesureHum();
           usleep(100000);
           mesureTem = lireMesureTem();
           mShm->lock(); // on prend la mémoire partagée
           strcpy(m_noMesBase, mesureTem);  // écriture dans la mémoire partagée
           strcpy(m_noMesBase+1, mesureHum);  // écriture dans la mémoire partagée
           mShm->unlock(); // on libère la mémmoire partagée
           sleep(1); // lecture toutes les s
       } // while
}

CCapteurTemHum_SI7021_A20::CCapteurTemHum_SI7021_A20(QObject *parent, int noMesBase) :
  QThread(parent)
{
  mShm = new CSharedMemory(this);
  connect(mShm, SIGNAL(sigErreur(QString)), this, SLOT(onErreur(QString)));
  mShm->memoireAttacher();

  mI2c = CI2c::getInstance(this, '1');
  connect(mI2c, SIGNAL(sigErreur(QString)), this, SLOT(onErreur(QString));
  m_fin=false;
  m_noMesBase = noMesBase;
  QDebug() << "Objet CCapteurTemHum_SI7021_A20 créé!";
}

CCapteurTemHum_SI7021_A20::~CCapteurTemHum_SI7021_A20()
{

  CI2c::freeInstance();
  mShm->detach();
  delete mShm;
  qDebug() << "Objet CCapteurTemHum_SI7021_A20 détruit !";
}

void CCapteurTemHum_SI7021_A20::onErreur(QString mess)
{
  emit sigErreur(mess);
}
